from moon.custom_image import CustomImage 
from moon.res.en.ui_messages import *
from datetime import datetime, timezone, timedelta
import urllib, urllib.request, json, sys, pkg_resources
from functools import lru_cache


# Get strings for URLs and IDs this Moon class will need
# https://stackoverflow.com/questions/60687577/trying-to-read-json-file-within-a-python-package
if sys.version_info >= (3, 7):
    import importlib.resources
    with importlib.resources.open_text("moon.res", "constants.json") as file:
        CONSTANTS_JSON_DICT = json.load(file)
else:
    import pkg_resources
    resource_package = __name__
    resource_path = '/'.join(('res', 'constants.json'))
    constants_string = pkg_resources.resource_string(resource_package, resource_path)
    CONSTANTS_JSON_DICT = json.loads(constants_string)
else:
    # todo this is a workaround when troubleshooting importing resources with package_data 
    CONSTANTS_JSON_DICT = {}

class Moon(CustomImage):
    def __init__(self, size=(1000,1000)):
        self.size = size
        self.SVS_ID_DICT = CONSTANTS_JSON_DICT["SVS_ID_DICT"]
        self.SVS_URL_BASE = CONSTANTS_JSON_DICT["SVS_URL_BASE"]
        self.SVS_JSON_URL_BASE = CONSTANTS_JSON_DICT["SVS_JSON_URL_BASE"]
        self.GITHUB_CONSTANTS_URL = CONSTANTS_JSON_DICT["GITHUB_CONSTANTS_URL"]
        super()
        return

    def __str__(self):
        return datetime.strftime(self.datetime,'%Y%m%d')

    def set_moon_phase(self, date=None, hour=None):
        try:
            self.set_moon_datetime(date, hour)
            self.request_moon_image()
            self.make_json_year_mooninfo_url()
            self.set_mooninfo_requested_year()
            self.set_mooninfo_requested_date()
        except Exception as e:
            raise e
        return True

    def set_moon_datetime(self, date=None, hour=None):
        """
        Keyword arguments:
        date -- UTC date in format YYYY-MM-DD, defaults to current date
        hour -- UTC hours 0 through 23, defaults to current hour
        """
        try:
            self.datetime = self.make_datetime(date, hour)
            self.image = None
            self.url = self.make_moon_image_url()
        except Exception as e:
            raise e
        return True

    def request_moon_image(self):
        try:
            self.image = self.set_image(url=self.url)
        except Exception as e:
            raise e
        return True

    def make_datetime(self, date, hour):
        # if hour and hour <= 0 or hour >= 23:
        #     raise ValueError(HOUR_ERROR)
        if hour is None:
            hour = datetime.now(timezone.utc).hour
        
        if date is None:
            date = datetime.now(timezone.utc)
        else:
            date = datetime.strptime(date, '%Y-%m-%d').replace(tzinfo=timezone.utc)
        
        year, month, day = date.year, date.month, date.day
  
        return datetime(year=year, month=month, day=day, hour=hour).replace(tzinfo=timezone.utc)
        
    def make_nasa_frame_id(self):
        #code logic courtesy of Ernie Wright
        year = self.datetime.year
        print("in make_nasa_frame_id:")
        print(self.datetime.year)
        print(self.svs_id)
        print()

        #todo - check why we were checking that the year isn't 2019
        # if (year != 2019):
        #     moon_imagenum = 1
        janone = datetime(year, 1, 1, 0, 0, 0, 0, tzinfo=timezone.utc )
        moon_imagenum = int(round((self.datetime - janone ).total_seconds() / 3600)) + 1

        #todo check why this was in here
        # if (moon_imagenum > 8760):
        #     moon_imagenum = 8760
        return str(moon_imagenum).zfill(4)

    def make_moon_image_url(self):
        try:
            self.svs_id = self.SVS_ID_DICT[str(self.datetime.year)]
            print("in make_moon_image_url:")
            print("self.datetime.year is ", self.datetime.year)
            print("self.svs_id is ", self.svs_id)
            print()
        except KeyError as e:
            years_available = sorted(self.SVS_ID_DICT.keys())
            requested_year = self.datetime.year
            if requested_year not in years_available:
                # try to get the ID from the github repo
                # in case the ID is available but the package hadn't been 
                # updated yet
                try:
                    response = urllib.request.urlopen(self.GITHUB_CONSTANTS_URL)
                    remote_json_dict = json.load(response)
                    self.SVS_ID_DICT = remote_json_dict["SVS_ID_DICT"]
                    self.svs_id = self.SVS_ID_DICT[str(self.datetime.year)]
                    print(UPDATED_PACKAGE_EXISTS_WITH_YEAR_ID.format(
                        year=requested_year
                        ))
                except:
                    # datetime wasn't found so unset it
                    self.datetime = None
                    raise KeyError(NO_SVS_ID_ERROR.format(
                        year=requested_year,
                        year_range_0=years_available[0],
                        year_range_1=years_available [-1]
                        ))
            else:
                raise e


        self.frame_id = self.make_nasa_frame_id()
        print("setting frame id: ", self.frame_id)
        return self.SVS_URL_BASE.format(
            year_id_modulo = str(self.svs_id - self.svs_id % 100),
            year_id = str(self.svs_id),
            frame_id = str(self.frame_id)
        )

    def save(self, prefix="moon-image-"):
        date = datetime.strftime(self.datetime,'%Y%m%d')
        self.save_to_disk(prefix + date)

    def get_moon_phase_date(self):
        return self.datetime

    def make_json_year_mooninfo_url(self):
        self.json_url = self.SVS_JSON_URL_BASE.format(
            year_id_modulo = str(self.svs_id - self.svs_id % 100),
            year_id = str(self.svs_id),
            frame_id = str(self.frame_id),
            year = self.datetime.year
        )

    @lru_cache()
    def set_mooninfo_requested_year(self):

        response = urllib.request.urlopen(self.json_url) 
        self.moon_year_info = json.loads(response.read())
        return self.moon_year_info

    def set_mooninfo_requested_date(self):
        try:
            print("going to try to update the moon_datetime_info: ")
            print(self.moon_datetime_info['time'])
            print(self.svs_id)
            print(self.get_moon_phase_date())
            print()
        except:
            print("no info yet")

        self.moon_datetime_info = self.moon_year_info[int(self.frame_id) - 1]
        print("just tried to update moon_datetime_info:")
        print(self.moon_datetime_info['time'])
        print(self.svs_id)
        print(self.get_moon_phase_date())
        print()







import unittest
from ..dialamoon import Moon
from ..terminal_ui import TerminalUi
from datetime import datetime
import numpy
import copy

class TestMoon(unittest.TestCase):
	def __init__(self, *args, **kwargs):
		super(TestMoon, self).__init__(*args, **kwargs)
		self.sm = Moon()

	# def test_returns_error_for_datetime_format_issue(self):
	# 	m = Moon()
	# 	self.assertRaises(ValueError, m.set_moon_datetime, "190-01-01")

	# def test_returns_helpful_error_for_datetime_range_issue(self):
	# 	m = Moon()
	# 	self.assertRaisesRegex(KeyError, r"Cannot find year .* in this version "\
	# 			"of the package. This package can get moons for years .* -"\
	# 			" .*. Please try a different year or check for an update "\
	# 			"for this package.",
	# 	 m.set_moon_datetime, "1900-01-01")

	# def test_makes_a_moon_image_url(self):
	# 	m = Moon()
	# 	m.set_moon_datetime(date="2019-01-01", hour=0)
	# 	assert m.url == "https://svs.gsfc.nasa.gov/vis/a000000/a004400/"\
	# 	"a004442/frames/730x730_1x1_30p/moon.0001.jpg"

	# def test_makes_a_json_data_url(self):
	# 	m = Moon()
	# 	m.set_moon_datetime("2019-01-01")
	# 	m.make_json_year_mooninfo_url()
	# 	assert m.json_url == "https://svs.gsfc.nasa.gov/vis/a000000/a004400/a004442/mooninfo_2019.json"

	# def test_gets_json_for_requested_datetime(self):
	# 	m = Moon()
	# 	m.set_moon_datetime(date="2019-01-01", hour=1)
	# 	m.make_json_year_mooninfo_url()
	# 	m.set_mooninfo_requested_year()
	# 	m.set_mooninfo_requested_date()
	# 	assert m.moon_datetime_info["time"] == '01 Jan 2019 01:00 UT'
		
	# def test_gets_moon_image_as_numpy_array(self):
	# 	m = Moon()
	# 	m.set_moon_datetime("2019-01-01")
	# 	m.request_moon_image()
	# 	self.assertIs(type(m.image), numpy.ndarray)

	# def test_takes_optional_hour_arg(self):
	# 	m = Moon()
	# 	m.set_moon_datetime(hour=1)
	# 	assert m.datetime.hour == 1

	# def test_can_get_last_hour_of_nonleap_year(self):
	# 	m = Moon()
	# 	m.set_moon_datetime(date="2019-12-31", hour=23)
	# 	m.request_moon_image()
	# 	m.make_json_year_mooninfo_url()
	# 	m.set_mooninfo_requested_year()
	# 	m.set_mooninfo_requested_date()

	# 	assert m.image_src == "https://svs.gsfc.nasa.gov/vis/a000000/a004400/a004442/"\
	# 	"frames/730x730_1x1_30p/moon.8760.jpg"

	# def test_can_get_last_hour_of_year(self):
	# 	m = Moon()
	# 	m.set_moon_datetime(date="2020-12-31", hour=23)
	# 	m.request_moon_image()
	# 	assert m.image_src == "https://svs.gsfc.nasa.gov/vis/a000000/a004700/a004768/"\
	# 	"frames/730x730_1x1_30p/moon.8784.jpg"

	# def test_can_get_moon_for_2022(self):
	# 	# note: this is just helpful for testing a version of `moon` where
	# 	# the SVS_ID has been added to the github repo but the user
	# 	# hasn't updated the package to include the new SVS_ID
	# 	m = Moon()
	# 	m.set_moon_datetime(date="2022-01-15")
	# 	m.request_moon_image()
	# 	assert m.SVS_ID_DICT["2022"] == 4955

	def test_gets_new_moon_info_if_another_year_requested(self):
		m = Moon()
		m.set_moon_phase("2019-01-15")
		# print(m.moon_datetime_info['time'])
		m.set_moon_phase("2020-01-16")
		# print(m.moon_datetime_info['time'])
		assert m.moon_datetime_info['time'] == '15 Jan 2020 09:00 UT'


if __name__ == '__main__':
	unittest.main()


from moon.dialamoon import Moon
import cv2
import matplotlib.pyplot as plt

class TerminalUi(Moon):
    def show(self):
        print("☽ The image will open up in a new window. ☽ "\
        	"\n☽ It might be behind your terminal window. ☽")
        try:
            plt.imshow(cv2.cvtColor(self.image, cv2.COLOR_BGR2RGB))
            plt.show()
            print("☽ Image closed. ☽")
        except:
            e = sys.exc_info()[1]
            print("Error: %s" % e) 



# import unittest
# from ..dialamoon import Moon
# from ..terminal_ui import TerminalUi
# from datetime import datetime
# from time import sleep
# import numpy
# import copy

# class TestMoonCache(unittest.TestCase):
#     def test_caches_json_year_data(self):
#         m = Moon()
#         m.set_moon_datetime(date="2019-01-01")
#         m.make_json_year_mooninfo_url()
#         time1 = datetime.now()
#         m.set_mooninfo_requested_year()
#         time2 = datetime.now()
#         sleep(5)
#         time3 = datetime.now()
#         m.set_mooninfo_requested_year()
#         time4 = datetime.now()
#         assert (time4 - time3).seconds == 0


#     def test_changes_moon_image_on_second_request(self):
#         m = Moon()
#         m.set_moon_datetime("2019-01-01")
#         m.request_moon_image()
#         url1 = m.url
#         im1 = copy.deepcopy(m.image)
#         m.set_moon_datetime("2020-01-01")
#         url2 = m.url
#         m.request_moon_image()
#         im2 = m.image
#         assert url1 != url2 and not numpy.array_equal(im1,im2)

#     def test_caches_moon_image_from_first_request(self):
#         m = TerminalUi()
#         m.set_moon_datetime("2020-01-06")
#         m.request_moon_image()
#         url1 = m.url
#         im1 = copy.deepcopy(m.image)
#         m.set_moon_datetime("2020-01-01")
#         url2 = m.url
#         m.request_moon_image()
#         im2 = copy.deepcopy( m.image)
#         m.set_moon_datetime("2020-01-06")
#         url3 = m.url
#         time2 = datetime.now()
#         m.request_moon_image()
#         time3 = datetime.now()
#         im3 = copy.deepcopy(m.image)
#         assert url1 == url3 and numpy.array_equal(im1,im3) and (time3 - time2).seconds == 0
#         assert numpy.array_equal(im1,im3)

#     def test_gets_json_year_data(self):
#         m = Moon()
#         m.set_moon_datetime("2019-01-01")
#         m.make_json_year_mooninfo_url()
#         m.set_mooninfo_requested_year()
#         assert m.moon_year_info[0]["time"] == '01 Jan 2019 00:00 UT'


#     def test_saves_time_getting_same_moon_image(self):
#         m = Moon()
#         m.set_moon_datetime("2019-01-01")
#         m.request_moon_image()
#         m.set_moon_datetime("2019-01-02")
#         m.request_moon_image()
#         time1 = datetime.now()
#         m.request_moon_image()
#         time2 = datetime.now()
#         assert (time2 - time1).seconds == 0

# if __name__ == '__main__':
#     unittest.main()




NO_SVS_ID_ERROR = "Cannot find year {year} in this version "\
                "of the package. This package can get moons for years {year_range_0} -"\
                " {year_range_1}. Please try a different year or check for an update "\
                "for this package."

UPDATED_PACKAGE_EXISTS_WITH_YEAR_ID = 	"Warning: There's a newer version of this `moon` package "\
										"that includes the requested year {year}. Your version "\
										"of this package has made an extra request to get the "\
										"ID for year {year} by requesting it from "\
										"the GitHub repo with the updated version of the "\
										"moon package. If you update this package, "\
										"it won't have to make this extra request next time."

HOUR_ERROR = "Hour is not between 0 and 23."



from distutils.core import setup
from setuptools import setup, find_packages

# long_description
readme_path = "./README.md"
long_description = open(readme_path, 'r').read()

setup(
    name = 'moon',
    packages = find_packages(),
    version = '1.1.7',
    license='MIT',
    description = 'Gets moon visualizations courtesy of SVS, NASA, Ernie Wright',
    long_description_content_type="text/markdown",
    long_description = long_description,
    author = 'Sadie Parker',
    author_email = 'sadiemparker@gmail.com',
    url = 'https://github.com/spacerest/moon',
    download_url = 'https://github.com/spacerest/moon/archive/v_1_1_7.tar.gz',
    keywords = ['MOON', 'ART'],
    install_requires=[
        'numpy~=1.16',
        'opencv-python~=4.5.4.60',
        'matplotlib~=3.2.1',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Other Audience',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    package_data={'constants': ['res/constants.json']},
    include_package_data=True
   
)



import urllib.request
import io
import cv2
import numpy as np
from copy import deepcopy, copy
from functools import lru_cache

class CustomImage():
    def __init__(self, img_size, **kwargs):
        self.size = img_size
        self.height = img_size[0]
        self.width = img_size[1]
        self.set_image(**kwargs)
        return

    def save_to_disk(self, filename):
        try:
            cv2.imwrite(filename + ".jpg", self.image)
        except Exception as e:
            print(e)

    def get_image(self):
        return self.image
    
    @lru_cache(maxsize=10)#$todo maybe revisit this maxsize
    def set_image(self, url=None, filename=None):
        # METHOD #1: OpenCV, NumPy, and urllib
	    # download the image, convert it to a NumPy array, and then read
	    # it into OpenCV format
        # https://www.pyimagesearch.com/2015/03/02/convert-url-to-image-with-python-and-opencv/
        if url:
            resp = urllib.request.urlopen(url)
            self.image_src = url
            self.image = np.asarray(bytearray(resp.read()), dtype="uint8")
            self.image = cv2.imdecode(self.image, cv2.IMREAD_COLOR)
            self.resize_image()
        elif filename:
            self.image_src = filename
            self.image = cv2.imread(filename)
            self.resize_image()
        #$todo not sure easy way to cache image
        #without returning the ndarray to child obj...?
        return self.image

    def resize_image(self):
        #https://medium.com/@manivannan_data/resize-image-using-opencv-python-d2cdbbc480f0
        #Preferable interpolation methods are cv.INTER_AREA for shrinking and cv.INTER_CUBIC(slow) & cv.INTER_LINEAR for zooming. By default, interpolation method used is cv.INTER_LINEAR for all resizing purposes.
        try:
            self.image = cv2.resize(self.image, self.size, interpolation=cv2.INTER_AREA)
        except Exception as e:
            raise e
        self.image = cv2.resize(self.image, self.size, interpolation=cv2.INTER_CUBIC)
